package com.bcxin.event.core.jdbc;

import com.bcxin.event.core.definitions.JdbcConnectorDefinition;
import com.bcxin.event.core.exceptions.BadEventException;
import com.zaxxer.hikari.HikariConfig;
import com.zaxxer.hikari.HikariDataSource;

import javax.sql.DataSource;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

public class JdbcMetaRepositoryBuilder {
    private final static Map<String, JdbcMetaRepository> repositoryMap = new ConcurrentHashMap<>();

    public static JdbcMetaRepository build(JdbcConnectorDefinition definition) {
        if (definition == null) {
            throw new BadEventException("JdbcConnectorDefinition 不该为null");
        }

        JdbcMetaRepository repository = repositoryMap.get(definition.getUniqueKey());
        if (repository == null) {
            synchronized (repositoryMap) {
                if (repository == null) {
                    HikariConfig config = new HikariConfig();
                    config.setJdbcUrl(definition.getUrl());
                    config.setUsername(definition.getUsername());
                    config.setPassword(definition.getPassword());
            /*
            config.addDataSourceProperty("cachePrepStmts", "true");
            config.addDataSourceProperty("prepStmtCacheSize", "250");
            config.addDataSourceProperty("prepStmtCacheSqlLimit", "2048");
             */
                    DataSource dataSource = new HikariDataSource(config);
                    repository = new JdbcMetaRepositoryImpl(dataSource);

                    repositoryMap.put(definition.getUniqueKey(), repository);
                }
            }
        }

        return repository;
    }
}
