package com.bcxin.event.core.jdbc;

import org.springframework.jdbc.core.PreparedStatementCreatorFactory;
import org.springframework.jdbc.core.SqlParameter;
import org.springframework.jdbc.core.namedparam.MapSqlParameterSource;
import org.springframework.jdbc.core.namedparam.NamedParameterUtils;
import org.springframework.jdbc.core.namedparam.ParsedSql;
import org.springframework.jdbc.core.namedparam.SqlParameterSource;

import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

public class JdbcNameParameterSqlParserImpl implements JdbcNameParameterSqlParser {
    public static final int DEFAULT_CACHE_LIMIT = 256;
    private volatile int cacheLimit = DEFAULT_CACHE_LIMIT;

    public int getCacheLimit() {
        return this.cacheLimit;
    }

    private final Map<String, ParsedSql> parsedSqlCache =
            new LinkedHashMap<String, ParsedSql>(DEFAULT_CACHE_LIMIT, 0.75f, true) {
                @Override
                protected boolean removeEldestEntry(Map.Entry<String, ParsedSql> eldest) {
                    return size() > getCacheLimit();
                }
            };

    @Override
    public ParseSqlParameter parse(String sql) {
        ParsedSql parsedSql = getParsedSql(sql);
        PreparedStatementCreatorFactory pscf = getPreparedStatementCreatorFactory(parsedSql,
                new MapSqlParameterSource() {
                });
        return ParseSqlParameter.create(parsedSql, pscf);
    }

    private ParsedSql getParsedSql(String sql) {
        if (getCacheLimit() <= 0) {
            return NamedParameterUtils.parseSqlStatement(sql);
        }

        synchronized (this.parsedSqlCache) {
            return this.parsedSqlCache.computeIfAbsent(sql, NamedParameterUtils::parseSqlStatement);
        }
    }

    protected PreparedStatementCreatorFactory getPreparedStatementCreatorFactory(
            ParsedSql parsedSql, SqlParameterSource paramSource) {
        String sqlToUse = NamedParameterUtils.substituteNamedParameters(parsedSql, paramSource);
        List<SqlParameter> declaredParameters = NamedParameterUtils.buildSqlParameterList(parsedSql, paramSource);
        return new PreparedStatementCreatorFactory(sqlToUse, declaredParameters);
    }
}
