package com.bcxin.event.core.jdbc;

import com.bcxin.event.core.exceptions.BadEventException;
import lombok.AccessLevel;
import lombok.Getter;
import org.springframework.jdbc.core.PreparedStatementCreatorFactory;
import org.springframework.jdbc.core.namedparam.ParsedSql;

import java.io.Serializable;
import java.lang.reflect.Field;
import java.util.List;

@Getter
public class ParseSqlParameter implements Serializable {
    private final ParsedSql parsedSql;
    private final PreparedStatementCreatorFactory pscf;

    public ParseSqlParameter(ParsedSql parsedSql, PreparedStatementCreatorFactory pscf) {
        this.parsedSql = parsedSql;
        this.pscf = pscf;
    }

    public static ParseSqlParameter create(ParsedSql parsedSql, PreparedStatementCreatorFactory pscf) {
        return new ParseSqlParameter(parsedSql, pscf);
    }

    @Getter(AccessLevel.PRIVATE)
    private List<String> _parameterNames;

    public List<String> getParameterNames() {
        if (_parameterNames == null) {
            _parameterNames = (List<String>) getFieldValue(this.getParsedSql(), "parameterNames");
        }

        return _parameterNames;
    }

    @Getter(AccessLevel.PRIVATE)
    private List<int[]> _parameterIndexes;
    public List<int[]> getParameterIndexes() {
        if (_parameterIndexes == null) {
            _parameterIndexes = (List<int[]>) getFieldValue(this.getParsedSql(), "parameterIndexes");
        }

        return _parameterIndexes;
    }

    @Getter(AccessLevel.PRIVATE)
    private Integer _totalParameterCount;
    private Integer getTotalParameterCount() {
        if (_totalParameterCount == null) {
            _totalParameterCount = (int) getFieldValue(this.getParsedSql(), "totalParameterCount");
        }

        return _totalParameterCount;
    }

    private Object getFieldValue(Object value,String fieldName)  {
        if(value==null) {
            return null;
        }

        try {
            Class tClass = value.getClass();
            Field field = tClass.getDeclaredField(fieldName);
            if (field == null) {
                return null;
            }

            field.setAccessible(true);
            return field.get(value);
        }
        catch (Exception ex) {
            throw new BadEventException(ex);
        }
    }
}
