package com.bcxin.event.core.jdbc.springutils;

import org.springframework.jdbc.core.namedparam.SqlParameterSourceUtils;
import org.springframework.jdbc.support.JdbcUtils;
import org.springframework.lang.Nullable;

public interface SqlParameterSource1 {

    /**
     * Constant that indicates an unknown (or unspecified) SQL type.
     * To be returned from {@code getType} when no specific SQL type known.
     * @see #getSqlType
     * @see java.sql.Types
     */
    int TYPE_UNKNOWN = JdbcUtils.TYPE_UNKNOWN;


    /**
     * Determine whether there is a value for the specified named parameter.
     * @param paramName the name of the parameter
     * @return whether there is a value defined
     */
    boolean hasValue(String paramName);

    /**
     * Return the parameter value for the requested named parameter.
     * @param paramName the name of the parameter
     * @return the value of the specified parameter
     * @throws IllegalArgumentException if there is no value for the requested parameter
     */
    @Nullable
    Object getValue(String paramName) throws IllegalArgumentException;

    /**
     * Determine the SQL type for the specified named parameter.
     * @param paramName the name of the parameter
     * @return the SQL type of the specified parameter,
     * or {@code TYPE_UNKNOWN} if not known
     * @see #TYPE_UNKNOWN
     */
    default int getSqlType(String paramName) {
        return TYPE_UNKNOWN;
    }

    /**
     * Determine the type name for the specified named parameter.
     * @param paramName the name of the parameter
     * @return the type name of the specified parameter,
     * or {@code null} if not known
     */
    @Nullable
    default String getTypeName(String paramName) {
        return null;
    }

    /**
     * Enumerate all available parameter names if possible.
     * <p>This is an optional operation, primarily for use with
     * {@link org.springframework.jdbc.core.simple.SimpleJdbcInsert}
     * and {@link org.springframework.jdbc.core.simple.SimpleJdbcCall}.
     * @return the array of parameter names, or {@code null} if not determinable
     * @since 5.0.3
     * @see SqlParameterSourceUtils#extractCaseInsensitiveParameterNames
     */
    @Nullable
    default String[] getParameterNames() {
        return null;
    }
}
